# code2beamer.rb
#
# Made by ReeB <cuissa_v@epita.fr>

require 'optparse'
require 'fileutils'
include FileUtils

$dir = "."
$file = nil
$line = 40
$generated = true

class Generator
  
  OPTIONS = "-c -f Courier7 --margin=0:0:0:0 --color --no-header --lines-per-page="
  COLUMN  = 50

  def initialize ( file, dir, line )
    @file = file
    @code_file = File.basename(file, ".code")
    @name = File.basename(@code_file, ".*")
    @path = File.dirname(@file)
    @dir = dir
    @tex_path = "#@dir/#@name.tex"
    @line = line
    ext = File.extname(@code_file)
    @ext = ext[1 .. ext.length - 1]

    @@generate_result = {
      "c"   => [ :generate_c,   "c"     ],
      "cpp" => [ :generate_cpp, "cpp"   ],
      "cc"  => [ :generate_cpp, "cpp"   ],
      "rb"  => [ :generate_rb,  "ruby"  ],
      "sh"  => [ :generate_sh,  "sh"    ],
      "pl"  => [ :generate_pl,  "perl"  ],
      "tig" => [ :generate_tig, "tig"   ]
    }

    # Check if dir exist
    mkdir @dir unless File.exist? @dir
  end

  def generate
    # Generate tex
    method(@@generate_result[@ext][0]).call
    @ext = @@generate_result[@ext][1]
    # Generate pdf
    generate_pdfs
  end

  def generate_pdfs
    f = File.open(".tmp", "w")
    i = 1
    # Skip # MAIN if found
    File.open(@file).each do |line|
      break if line =~ /^(\/\/|\/*|\#) EXCLUDE/
      if line =~ /^(\/\/|\/\*|\#) CUT/
        f.close
        generate_pdf "#@name-#{i}"
        i += 1
        f = File.open(".tmp", "w")
        next
      end
      f << line
    end
    f.close unless f.closed?
    name = i == 1 ? @name : "#@name-#{i}"

    generate_pdf name
  end

  def generate_pdf ( name )
    system("enscript #{OPTIONS}#@line -E#@ext -q -o #{name}.eps .tmp && rm .tmp")
    system("epstopdf #{name}.eps && rm -f #{name}.eps")
    system("mv #{name}.pdf #@dir/")
  end
  
  def tex_header ( value )
    File.open(@tex_path, "w") do |f|
      f << "\\begin{tiny}\\begin{verbatim}\n"
      f << "$ #{value}\n"
    end
  end

  def tex_footer
    def noncut_line ( txt )
      begin
        size = txt.length > COLUMN ? COLUMN : txt.length
        yield txt[0..size] + (size == COLUMN ? "\n" : "")
        txt = size == COLUMN ? txt[size .. txt.length - 1] : nil
      end until txt.nil?
    end

    File.open(@tex_path, "a+") do |f|
      File.open("#@tex_path.err", "r").each do |line|
        noncut_line(line) do |l|
          f << "[STDERR] " << l
        end
      end
      f << "\\end{verbatim}\\end{tiny}\n"
    end
    rm_f "#@tex_path.err"
  end

  def generate_cmd ( cmd, value = "" )
    value = "#{cmd} #@code_file" if value == ""
    tex_header(value)
    Kernel.fork do
      STDOUT.reopen(@tex_path, 'a+')
      STDERR.reopen("#@tex_path.err", 'w')
      exec("#{cmd} #@file")
    end
    Process.waitpid
    tex_footer
  end

  def generate_cpp
    File.open("#@path/#@code_file", "w") do |f|
      File.open(@file).each { |line|  f << line }
    end
    generate_cmd("g++ #@path/#@code_file && ./a.out && rm a.out && true #", "./#@name")
    rm_f "#@path/#@code_file"
  end
  alias :generate_c :generate_cpp

  def generate_rb
    generate_cmd("ruby")
  end

  def generate_sh
    generate_cmd("sh")
  end

  def generate_pl
    generate_cmd("perl")
  end
end

# Options
opt = OptionParser.new;

opt.on("-h", "--help", "print the usage") do
  puts opt
  exit 0
end

opt.on("-d", "--dir DIR", "output directory") do |dir|
  $dir = dir
end

opt.on("-l", "--line-number N", "Number of line in one page") do |line|
  $line = line
end

rest = opt.parse(ARGV)
$file = rest[0] if rest.length

if $file.nil?
  puts opt
  exit 1
end

#
# Main
#

pdf_file = $file.gsub /\..+\.code/, '.pdf'
pdf_file.gsub! /code\//, 'generated/'
pdf_one = pdf_file.gsub /\.pdf/, '-1.pdf'

# Check if pdf's must be regenerated
begin
  exit 0 if File.mtime(pdf_file) > File.mtime($file) 
rescue
end
begin
  exit 0 if File.mtime(pdf_one) > File.mtime($file)
rescue
end

puts "Generate #{pdf_file}"
generator = Generator.new( $file, $dir, $line )
generator.generate

